
function [x,fVal,niter,rCode] = newtonsys(f, x0, tol, maxiter,showIts, varargin)
% NEWTONSYS Newton's method for a system of equations.
%    
% Input parameters:
%   f: Name of model function as a string. The function must be
%      defined in the following way: the first argument must take a
%      1xn column vector with the values x_1, x_2, ..., x_n for
%      the evalation of f. The second argument of the function must
%      accept a FLAG. If FLAG is empty, the function must be
%      evaluated regularly, and the result must be returned as a
%      colunm vector. If the FLAG is set to 'jacobian', the
%      function must return the evaluation result of the jacobian
%      matrix instead (as nxn matrix). After the FLAG parameter,
%      the function may specify an arbitrary number of additional
%      parameters. See below for an example model function
%      definition.
%   x0: initial guess, a 1xn column vector.
%   tol: Error tolerance for the relative increment of x.
%   maxiter: maximum number of iterations to perform.
%   p1, p2, ..., pn: An arbitrary number of parameters that are
%                    passed on to the function f when evaluated.
%
% Output paramters:
%         x: Approximation to the solution, 1xn column vector.
%   niter: Number of iterations performed.
%return code 1 for success 0 for fail
%
%      Example for calling newton:
%        [x, niter] = newton('robotarm', [pi/2; pi/2], 1e-5, ...
%                            20, [4; 2], [3; 4])
%
%      Example for the definition of the function f:
%        function out = robotarm(x, flag, l, p)
%        if isempty(flag)
%          flag = '';
%        end
%        switch lower(flag)
%         case ''
%                out = [l(1)*cos(x(1))-l(2)*cos(x(1)+x(2))-p(1); ...
%                 l(1)*sin(x(1))-l(2)*sin(x(1)+x(2))-p(2)];     
%         case 'jacobian'
%                out = [-l(1)*sin(x(1))+l(2)*sin(x(1)+x(2)), ...
%                 l(2)*sin(x(1)+x(2)); ...
%                 l(1)*cos(x(1))+l(2)*cos(x(1)+x(2)), ...
%                 -l(2)*cos(x(1)+x(2))];        
%        end
%
% Note: The newton method will fail if the determinant of the
%       jacobian matrix becomes zero.


        
% Author : Andreas Klimke, Universitt Stuttgart
% Version: 1.0
% Date   : June 13, 2003


% Initialization

fPrev='not set';
xPrev='not set';


rCode=0;
x = x0;
x_old = x;
niter = 0;
descTol=1e-6;

try
rc=1;
for k = 1:maxiter
AA=feval(f, x, 'jacobian', varargin{:});
bb=feval(f, x, [], varargin{:});
f_old=bb;
encounterdBadVals(showIts,bb,xPrev,fPrev);
[mm,nn]=size(AA);
[QQ,RR,PP]=qr(AA); % full QR factorization
%rr=rank(AA); % could also get rank directly from QR factorization ...
theDiag=diag(RR);
rr=sum(abs(diag(RR))>0);
% construct the submatrices
Q1=QQ(:,1:rr);
Q2=QQ(:,rr+1:mm);
R1=RR(1:rr,1:rr);
% check if b is in range(AA)

% if this is zero or very small, b is in range(A)
if(norm(Q2'*bb)>descTol)
%'newton sys having problem with rank of derivative matrix'
end
% construct a solution
xx=PP*[R1\(Q1'*bb); zeros(mm-rr,1)]; % satisfies Ax=b, if b is in range(A)
% check alleged solution (just to be sure)
if(norm(AA*xx-bb)>descTol);
%'newton sys having problem with rank of derivative matrix'
end
x=x-xx;
        % Iteration step
%        x = x - AA \ ...
 %
                       bb;
        niter = niter + 1;
                
        % Break condition
        if norm(x) < eps
                if norm(x-x_old) <= tol
rCode=1;
fVal=feval(f, x, [], varargin{:});
                        return
                end
        else
%                if and(norm(x-x_old)/norm(x) <= tol,norm(f_old)<=tol)
                if (norm(f_old)<=tol)
rCode=1;
fVal=feval(f, x, [], varargin{:});
                        return
			end
        end
x_prev=x_old;f_prev=f_old;        
        x_old = x;
end
error('newtonsys: Maximum number of iterations reached.');
fVal=feval(f, x, [], varargin{:});
reportBadEqns(showIts,bb);
rc=0;
catch
fVal=feval(f, x, [], varargin{:});
reportBadEqns(showIts,bb);
rc=0;
end        


function encounterdBadVals(showIts,fVal,xPrev,fPrev)
if(showIts)
realBig=find((fVal==inf)+(fVal==-inf));
realBad=find(isnan(fVal));
if(length(realBig)>0)
realBig
fPrev;
xPrev;
error('infinity for some equations')
end
if(length(realBad)>0)
realBad
error('NaN for some equations: quitting newtonsys');
end
end

function reportBadEqns(showIts,fVal)
if(showIts)
realBig=find((fVal==inf)+(fVal==-inf));
realBad=find(isnan(fVal));
if(length(realBig)>0)
'infinity for some equations'
realBig
end
if(length(realBad)>0)
'NaN for some equations'
realBad
end
[toFindBig,idx]=sort(abs(fVal),1,'descend');
aTenth=max(1,min(10,round(length(fVal)/10)));
disp('largest discrepancies and their locations')
disp(toFindBig(1:aTenth))
disp(idx(1:aTenth))
end
